<?php

namespace Drupal\Tests\depcalc\Kernel\EventSubscriber\DependencyCollector;

use Drupal;
use Drupal\Core\Entity\EntityInterface;
use Drupal\depcalc\DependencyStack;
use Drupal\depcalc\DependentEntityWrapper;
use Drupal\image\Entity\ImageStyle;
use Drupal\KernelTests\KernelTestBase;
use Drupal\responsive_image\Entity\ResponsiveImageStyle;

/**
 * Class ConfigEntityDependencyCollector.
 *
 * @group depcalc
 *
 * @package Drupal\Tests\depcalc\Kernel\EventSubscriber\DependencyCollector
 *
 * @covers \Drupal\depcalc\EventSubscriber\DependencyCollector\ConfigEntityDependencyCollector
 */
class ConfigEntityDependencyCollectorTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'breakpoint',
    'depcalc',
    'image',
    'path_alias',
    'responsive_image',
    'system',
    'taxonomy',
  ];

  /**
   * Calculates all the dependencies of a given entity.
   *
   * @var \Drupal\depcalc\DependencyCalculator
   */
  private $calculator;

  /**
   * {@inheritDoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->installEntitySchema('path_alias');
    $this->calculator = Drupal::service('entity.dependency.calculator');
  }

  /**
   * Tests config entities dependency calculation.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Exception
   */
  public function testConfigEntitiesDependencyCalculation() {
    // Creating a new dependent image style.
    $style1 = $this->createImageStyle('dummy_image_style_1');
    $wrapper = new DependentEntityWrapper($style1);
    $dependencies = $this->calculator->calculateDependencies($wrapper, new DependencyStack());

    $this->assertNotEmpty($dependencies);
    $this->assertEmpty($dependencies['module']);
    $this->assertEqual(2, count($dependencies));
    $this->assertArrayHasKey($style1->uuid(), $dependencies);

    // Creating a one more dependent image style.
    $style2 = $this->createImageStyle('dummy_image_style_2');

    // Creating a responsive image style which depends on the above styles.
    /** @var \Drupal\responsive_image\ResponsiveImageStyleInterface $responsiveImageStyle */
    $responsiveImageStyle = ResponsiveImageStyle::create([
      'id' => 'dummy_responsive_image_style_1',
      'label' => 'Dummy responsive image style 1',
    ]);
    $responsiveImageStyle->addImageStyleMapping('dummy_breakpoint_1', '1x', [
      'image_mapping_type' => 'image_style',
      'image_mapping' => 'dummy_image_style_1',
    ]);
    $responsiveImageStyle->addImageStyleMapping('dummy_breakpoint_2', '2x', [
      'image_mapping_type' => 'image_style',
      'image_mapping' => 'dummy_image_style_2',
    ]);
    $responsiveImageStyle->save();

    $wrapper = new DependentEntityWrapper($responsiveImageStyle);
    $dependencies = $this->calculator->calculateDependencies($wrapper, new DependencyStack());
    $this->assertNotEmpty($dependencies);
    $this->assertEqual(4, count($dependencies));
    $this->assertArrayHasKey($style1->uuid(), $wrapper->getDependencies());
    $this->assertArrayHasKey($style2->uuid(), $wrapper->getDependencies());

    $wrapper->addModuleDependencies(['image']);
    $dependencies = $this->calculator->calculateDependencies($wrapper, new DependencyStack());
    $this->assertEqual($dependencies['module'], ['image']);
  }

  /**
   * Creates a dummy image style.
   *
   * @param string $name
   *   Image style name.
   *
   * @return \Drupal\image\ImageStyleInterface
   *   Image style.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function createImageStyle(string $name): EntityInterface {
    $style = ImageStyle::create([
      'name' => $name,
      'label' => $this->randomString(),
    ]);
    $style->save();
    return $style;
  }

}
